/**
 * FlpTime(Flappy Time)
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * drawclock.c(draw clock) 
 */

WinHandle screenBufferH;
RectangleType sourceBounds;

#define SCREEN_WIDTH 160
#define SCREEN_HEIGHT 160

/**
 * draw bitmap
 */
static void drawBitmap(int id, int x, int y) {
  VoidHand objectBitmapHandles;
  BitmapPtr objectBitmapPtr;

  objectBitmapHandles = DmGetResource(bitmapRsc, id);
  objectBitmapPtr = MemHandleLock(objectBitmapHandles);
  WinDrawBitmap(objectBitmapPtr, x, y);
  MemPtrUnlock(objectBitmapPtr);
  DmReleaseResource(objectBitmapHandles);
}

/**
 * draw letters(12, 3, 6, 9) and dots on clock
 */
static void drawClockBackGround() {
  drawBitmap(TWELVE_BITMAP, 68, 1);
  drawBitmap(THREE_BITMAP, 143, 72);
  drawBitmap(SIX_BITMAP, 72, 143);
  drawBitmap(NINE_BITMAP, 1, 72);
  drawBitmap(ONE_BITMAP, 80+46-8, 1);
  drawBitmap(TWO_BITMAP, 151, 80-46);
  drawBitmap(FOUR_BITMAP, 151, 80+46-8);
  drawBitmap(FIVE_BITMAP, 80+46-8, 151);
  drawBitmap(SEVEN_BITMAP, 80-46, 151);
  drawBitmap(EIGHT_BITMAP, 1, 80+46-8);
  drawBitmap(TEN_BITMAP, 1, 80-46);
  drawBitmap(ELEVEN_BITMAP, 80-46, 1);
}

/**
 * create offscreen buffer
 */
static void createScreenBuffer() {
  Word error;
  WinHandle oldDrawWinH;

  screenBufferH = WinCreateOffscreenWindow
    (SCREEN_WIDTH, SCREEN_HEIGHT, screenFormat, &error);
  sourceBounds.topLeft.x = sourceBounds.topLeft.y = 0;
  sourceBounds.extent.x = SCREEN_WIDTH;
  sourceBounds.extent.y = SCREEN_HEIGHT;
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);
  WinEraseRectangle(&sourceBounds, 0);
  drawClockBackGround();
  WinSetDrawWindow(oldDrawWinH);
}

/*
 * draw and erase lines
 */
#define THICK_LINE_MAX 18

ThickLine thickLine[THICK_LINE_MAX];
int tlCnt = 0;

static void eraseThickLine() {
  int i;
  for ( i=0 ; i<tlCnt ; i++ ) {
    ThickLine tl = thickLine[i];
    WinEraseLine(tl.x1, tl.y1, tl.x2, tl.y2);
    if ( tl.hrzt ) {
      WinEraseLine(tl.x1+1, tl.y1, tl.x2+1, tl.y2);
    } else {
      WinEraseLine(tl.x1, tl.y1+1, tl.x2, tl.y2+1);
    }
  }
  tlCnt = 0;
}

#define SHIFT_WIDTH 3
#define SHIFT_WIDTH_HALF 1

static void drawThickLine(int sx, int sy, int dx, int dy) {
  //int rx, ry;
  Boolean hr = FALSE;
  if ( abs(sx-dx) < abs(sy-dy) ) {
    hr = TRUE;
  }
  /*sx += randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF; 
  sy += randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF;
  dx += randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF;
  dy += randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF;
  rx = (sx+dx)/2 + randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF;
  ry = (sy+dy)/2 + randN(SHIFT_WIDTH)-SHIFT_WIDTH_HALF;*/
  //WinDrawLine(sx, sy, rx, ry);
  //WinDrawLine(sx, sy, rx, ry);
  WinDrawLine(sx, sy, dx, dy);
  if ( hr ) {
    //WinDrawLine(sx+1, sy, rx+1, ry);
    //WinDrawLine(rx+1, ry, dx+1, dy);
    WinDrawLine(sx+1, sy, dx+1, dy);
  } else {
    //WinDrawLine(sx, sy+1, rx, ry+1);
    //WinDrawLine(rx, ry+1, dx, dy+1);
    WinDrawLine(sx, sy+1, dx, dy+1);
  }
  thickLine[tlCnt].x1 = sx; thickLine[tlCnt].y1 = sy;
  //thickLine[tlCnt].x2 = rx; thickLine[tlCnt].y2 = ry;
  thickLine[tlCnt].x2 = dx; thickLine[tlCnt].y2 = dy;
  thickLine[tlCnt].hrzt = hr;
  tlCnt++;
  /*thickLine[tlCnt].x1 = rx; thickLine[tlCnt].y1 = ry;
  thickLine[tlCnt].x2 = dx; thickLine[tlCnt].y2 = dy;
  thickLine[tlCnt].hrzt = hr;
  tlCnt++;*/
}

/*
 * draw clock hands
 */
#define X_CLOCK_CENTER 80
#define Y_CLOCK_CENTER 80

int handLength[] = {40, 60, 50};
int handWidth[] = {10, 5, 8};

int x1, y1, x2, y2, x3, y3;

static void calcHandPosition(int hr, int length, int width) {
  x1 = X_CLOCK_CENTER + isin(hr)*length/256;
  y1 = X_CLOCK_CENTER - icos(hr)*length/256;
  x2 = X_CLOCK_CENTER + isin(hr+80)*width/256;
  y2 = X_CLOCK_CENTER - icos(hr+80)*width/256;
  x3 = X_CLOCK_CENTER + isin(hr-80)*width/256;
  y3 = X_CLOCK_CENTER - icos(hr-80)*width/256;
}

static void calcSecondHandPosition(int hr, int length, int width) {
  x1 = X_CLOCK_CENTER + isin(hr)*length/256;
  y1 = X_CLOCK_CENTER - icos(hr)*length/256;
  x2 = X_CLOCK_CENTER + isin(hr+120)*width/256;
  y2 = X_CLOCK_CENTER - icos(hr+120)*width/256;
}

static void drawHand() {
  drawThickLine(x1, y1, x2, y2);
  drawThickLine(x2, y2, x3, y3);
  drawThickLine(x3, y3, x1, y1);
}

static void drawSecondHand() {
  drawThickLine(x1, y1, x2, y2);
}

static void drawClock() {
  int i, j;
  WinHandle oldDrawWinH;
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);

  eraseThickLine();
  for ( i=0 ; i<2 ; i++ ) {
    calcHandPosition(handStatus[i].deg, handLength[i], handWidth[i]);
    drawHand();
  }
  calcSecondHandPosition(handStatus[2].deg, handLength[2], handWidth[2]);
  drawSecondHand();
  WinSetDrawWindow(oldDrawWinH);
}

/**
 * draw offscreen buffer to display
 */
static void flipDisplay() {
  WinCopyRectangle
    (screenBufferH, 0, &sourceBounds, 0, 0, scrCopy);
} 
